/*   FILE: lcddaap.vala -- A simple DAAP music player
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 27 November 2013
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using Gst;
using Pifacecad;
using Posix;

const uint LIGHTS_OUT_SECONDS = 10;
const uint SWITCH_POLL_MILLISECONDS = 1;

private class LCDPlayer {
	public Stack<State> stateStack { get; set; default = null; }
	public dynamic Gst.Element element;

	private MainLoop loop;
	private Gee.ArrayList<DMAP.MdnsBrowserService> services = new Gee.ArrayList<unowned DMAP.MdnsBrowserService> ();
	private DMAP.MdnsBrowser browser;
	private DMAP.Connection connection;
	private SimpleDMAPDb db;
	private SimpleDAAPRecordFactory factory;

	private bool connected_cb (DMAP.Connection connection, bool result, string? reason) {
		// FIXME: Handle connections that require authentication.

		stateStack.pop (); // Pop dummy "Connecting to service" state.
		stateStack.push ( new StateChooseCategory (loop, this, db));

		return true;
	}

	private void service_added_cb (DMAP.MdnsBrowserService *service) {
		services.add (service);
		stateStack.peek ().print_selected ();
	}

	public void connect (DMAP.MdnsBrowserService *service) {
		db         = new SimpleDMAPDb ();
		factory    = new SimpleDAAPRecordFactory ();
		connection = (DMAP.Connection) new DAAP.Connection (service->service_name,
                                                                    service->host,
                                                                    service->port,
                                                                    db,
                                                                    factory);
		
		connection.connect (connected_cb);
	}

	public LCDPlayer () {
		loop    = new MainLoop ();
		browser = new DMAP.MdnsBrowser (DMAP.MdnsBrowserServiceType.DAAP);
		element = Gst.ElementFactory.make ("playbin", "play");

		stateStack = new Stack<State> ();

		// FIXME: Close LCD on exit.
		Pifacecad.open ();
		Pifacecad.lcd_backlight_on ();

		Timeout.add_seconds (LIGHTS_OUT_SECONDS, () => {
			Pifacecad.lcd_backlight_off (); return false;
		});

		browser.service_added.connect (service_added_cb);

		try {
			browser.start ();
		} catch (Error e) {
			GLib.error ("IOChannelError: %s\n", e.message);
		}

		stateStack.push (new StateChooseService (loop, this, services));

		loop.run ();
	}
}

int main (string[] args) {     
	Gst.init (ref args);

	Posix.termios old_tio, new_tio;
	Posix.tcgetattr(Posix.STDIN_FILENO, out old_tio);
	new_tio = old_tio;
	new_tio.c_lflag &=(~ICANON & ~ECHO);
	Posix.tcsetattr(STDIN_FILENO, TCSANOW, new_tio);

	new LCDPlayer ();

	Posix.tcsetattr(STDIN_FILENO, TCSANOW, old_tio);

	return 0;
}
