/*   FILE: state-choose-album.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class StateChooseAlbum: State {
	private MainLoop loop;
	private Gee.ArrayList<string> albums = new Gee.ArrayList<string> ();
	private Gee.HashSet<string> album_set = new Gee.HashSet<string> ();
	private int current = 0;
	private DMAP.Db db;

	public StateChooseAlbum (MainLoop loop, LCDPlayer player, DMAP.Db db) {
		this.loop   = loop;
		this.player = player;
		this.db     = db;

		db.foreach ((k, v) => {
			album_set.add (((SimpleDAAPRecord) v).songalbum);
		});

		foreach (string s in album_set) {
			albums.add (s);
		}

		albums.sort ();

		print_selected ();
	}

	// Print selected item.
	public override void print_selected () {
		output (albums[current]);
	}

	// Select next menu item.
	public override void next () {
		current = (current + 1) % albums.size;
		print_selected ();
	}

	// Select previous menu item.
	public override void previous () {
		current = (current + albums.size - 1) % albums.size;
		print_selected ();
	}

	// Transition to next state, based on current selection.
	public override State transition () {
		Gee.ArrayList<int> playlist = new Gee.ArrayList<int> ();

		db.foreach ((k, v) => {
			if (albums[current] == ((SimpleDAAPRecord) v).songalbum) {
				playlist.add ((int) k);
			}
		});

		playlist.sort ((id1, id2) => {
			SimpleDAAPRecord record1 = (SimpleDAAPRecord) db.lookup_by_id (id1);
			SimpleDAAPRecord record2 = (SimpleDAAPRecord) db.lookup_by_id (id2);
			return record1.track - record2.track;
		});

		return new StatePlay (loop, player, db, playlist);
	}
}
