/*   FILE: state-choose-artist.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 06 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class StateChooseArtist : State {
	private MainLoop loop;
	private Gee.ArrayList<string> artists = new Gee.ArrayList<string> ();
	private Gee.HashSet<string> artist_set = new Gee.HashSet<string> ();
	private int current = 0;
	private DMAP.Db db;

	public StateChooseArtist (MainLoop loop, LCDPlayer player, DMAP.Db db) {
		this.loop   = loop;
		this.player = player;
		this.db     = db;


		db.foreach ((k, v) => {
			artist_set.add (((SimpleDAAPRecord) v).songartist);
		});

		foreach (string s in artist_set) {
			artists.add (s);
		}

		artists.sort ();

		print_selected ();
	}

	// Print selected item.
	public override void print_selected () {
		output (artists[current]);
	}

	// Select next menu item.
	public override void next () {
		current = (current + 1) % artists.size;
		print_selected ();
	}

	// Select previous menu item.
	public override void previous () {
		current = (current + artists.size - 1) % artists.size;
		print_selected ();
	}

	// Transition to next state, based on current selection.
	public override State transition () {
		Gee.ArrayList<int> playlist = new Gee.ArrayList<int> ();

		db.foreach ((k, v) => {
			if (artists[current] == ((SimpleDAAPRecord) v).songartist) {
				playlist.add ((int) k);
			}
		});

		playlist.sort ((id1, id2) => {
                        SimpleDAAPRecord record1 = (SimpleDAAPRecord) db.lookup_by_id (id1);
                        SimpleDAAPRecord record2 = (SimpleDAAPRecord) db.lookup_by_id (id2);

			if (record1.songalbum == record2.songalbum) {
				return record1.track - record2.track;
			} else {
				if (record1.songalbum == record2.songalbum) {
					return 0;
				} else if (record1.songalbum < record2.songalbum) {
					return -1;
				} else {
					return 1;
				}
			}
                });

		return new StatePlay (loop, player, db, playlist);
	}
}
