/*   FILE: state-choose-category.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class StateChooseCategory : State {
	private MainLoop loop;
	private Gee.ArrayList<string> categories = new Gee.ArrayList<string> ();
	private int current = 0;
	private DMAP.Db db;

	public StateChooseCategory (MainLoop loop, LCDPlayer player, DMAP.Db db) {
		this.loop   = loop;
		this.player = player;
		this.db     = db;

		categories.add ("Album");
		categories.add ("Artist");
		categories.add ("Title");

		print_selected ();
	}

	// Print selected item.
	public override void print_selected () {
		output (categories[current]);
	}

	// Select next menu item.
	public override void next () {
		current = (current + 1) % categories.size;
		print_selected ();
	}

	// Select previous menu item.
	public override void previous () {
		current = (current + categories.size - 1) % categories.size;
		print_selected ();
	}

	// Transition to next state, based on current selection.
	public override State transition () {
		if ("Album" == categories[current]) {
			return new StateChooseAlbum (loop, player, db);
		} if ("Artist" == categories[current]) {
			return new StateChooseArtist (loop, player, db);
		} if ("Title" == categories[current]) {
			return new StateChooseTitle (loop, player, db);
		} else {
			GLib.assert_not_reached ();
		}
	}
}
