/*   FILE: state-choose-service.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

class StateChooseService : State {
	private MainLoop loop;
	private Gee.ArrayList<unowned DMAP.MdnsBrowserService> services;
	private int current = 0;

	public StateChooseService (MainLoop loop, LCDPlayer player, Gee.ArrayList<unowned DMAP.MdnsBrowserService> services) {
		this.loop     = loop;
		this.services = services;
		this.player   = player;

		print_selected ();
	}

	// Print selected item.
	public override void print_selected () {
		if (0 == services.size) {
			output ("No share found");
		} else {
			output (services[current].service_name);
		}
	}

	// Select next menu item.
	public override void next () {
		current = services.size > 0 ? (current + 1) % services.size : 0;
		print_selected ();
	}

	// Select previous menu item.
	public override void previous () {
		current = services.size > 0 ? (current + services.size - 1) % services.size : 0;
		print_selected ();
	}

	// Transition to next state, based on current selection.
	public override State transition () {
		unowned DMAP.MdnsBrowserService service = services[current];

		return new StateConnecting (loop, player, service);
	}
}
